﻿using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Resources;
using System.Xml.Serialization;
using Microsoft.Phone;
using Microsoft.Phone.BackgroundTransfer;
using Microsoft.Phone.Controls;

namespace BigPicture
{
    public partial class MainPage : PhoneApplicationPage
    {
        const string transferDirectory = "/shared/transfers";   // Required for background transfer
        const string imagesDirectory = "/images";
        const string thumbnailsDirectory = PictureInfo.ThumbnailsDirectory;

        PictureLibrary pictureLibrary;
        TransferPreferences transferPreferences = TransferPreferences.None;

        public MainPage()
        {
            InitializeComponent();

            // Load picture library information for ListBox
            Uri uri = new Uri("PictureLibrary.xml", UriKind.Relative);
            StreamResourceInfo resourceInfo = Application.GetResourceStream(uri);
            StreamReader reader = new StreamReader(resourceInfo.Stream);
            XmlSerializer xml = new XmlSerializer(typeof(PictureLibrary));
            pictureLibrary = xml.Deserialize(reader) as PictureLibrary;
            listBox.ItemsSource = pictureLibrary.Pictures;

            using (IsolatedStorageFile isoStorage = IsolatedStorageFile.GetUserStoreForApplication())
            {
                // Make sure directories exist
                if (!isoStorage.DirectoryExists(transferDirectory))
                    isoStorage.CreateDirectory(transferDirectory);

                if (!isoStorage.DirectoryExists(imagesDirectory))
                    isoStorage.CreateDirectory(imagesDirectory);

                if (!isoStorage.DirectoryExists(thumbnailsDirectory))
                    isoStorage.CreateDirectory(thumbnailsDirectory);

                // Get files already downloaded
                string[] filenames = isoStorage.GetFileNames(imagesDirectory + "/*.*");

                foreach (PictureInfo pictureInfo in pictureLibrary.Pictures)
                    pictureInfo.IsDownloaded = filenames.Contains<string>(pictureInfo.Filename);
            }
        }

        protected override void OnNavigatedTo(NavigationEventArgs args)
        {
            // Might be returning from TransferSettingsPage
            IsolatedStorageSettings applicationSettings = IsolatedStorageSettings.ApplicationSettings;

            if (applicationSettings.Contains(TransferSettingsPage.Key))
                transferPreferences = (TransferPreferences)applicationSettings[TransferSettingsPage.Key];
            
            // Get current requests
            foreach (BackgroundTransferRequest request in BackgroundTransferService.Requests)
            {
                int index = Int32.Parse(request.Tag);
                PictureInfo pictureInfo = pictureLibrary.Pictures[index];
                pictureInfo.DownloadRequestId = request.RequestId;

                if (request.TransferStatus == TransferStatus.Completed)
                {
                    TransferCompleted(pictureInfo, request);
                }
                else
                {
                    pictureInfo.IsDownloading = true;
                    pictureInfo.DownloadProgress = 100.0 * request.BytesReceived / request.TotalBytesToReceive;
                    pictureInfo.DownloadStatus = FormatTransferStatus(request.TransferStatus);
                    request.TransferProgressChanged += OnBackgroundTransferRequestProgressChanged;
                    request.TransferStatusChanged += OnBackgroundTransferRequestStatusChanged;
                }
            }
            base.OnNavigatedTo(args);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs args)
        {
            foreach (BackgroundTransferRequest request in BackgroundTransferService.Requests)
            {
                int index = Int32.Parse(request.Tag);
                PictureInfo pictureInfo = pictureLibrary.Pictures[index];

                // Remove event handlers
                request.TransferProgressChanged -= OnBackgroundTransferRequestProgressChanged;
                request.TransferStatusChanged -= OnBackgroundTransferRequestStatusChanged;
            }
            base.OnNavigatedFrom(args);
        }

        void OnDownloadButtonClick(object sender, RoutedEventArgs args)
        {
            Button btn = sender as Button;
            PictureInfo pictureInfo = btn.DataContext as PictureInfo;

            Uri requestUri = new Uri(pictureInfo.Url);
            Uri locationUri = new Uri(transferDirectory + "/" + pictureInfo.Filename, UriKind.Relative);
            BackgroundTransferRequest request = new BackgroundTransferRequest(requestUri, locationUri);
            request.Method = "GET";
            request.Tag = pictureLibrary.Pictures.IndexOf(pictureInfo).ToString();
            request.TransferPreferences = transferPreferences;
            request.TransferProgressChanged += OnBackgroundTransferRequestProgressChanged;
            request.TransferStatusChanged += OnBackgroundTransferRequestStatusChanged;

            try
            {
                BackgroundTransferService.Add(request);
                pictureInfo.IsDownloading = true;
                pictureInfo.DownloadProgress = 0;
                pictureInfo.DownloadRequestId = request.RequestId;
            }
            catch (Exception exc)
            {
                MessageBox.Show("Problem with download request: "+ exc.Message,
                                "Big Picture", MessageBoxButton.OK);
            }
        }

        void OnCancelDownloadButtonClick(object sender, RoutedEventArgs args)
        {
            Button btn = sender as Button;
            PictureInfo pictureInfo = btn.DataContext as PictureInfo;
            string requestId = pictureInfo.DownloadRequestId;
            BackgroundTransferRequest request = BackgroundTransferService.Find(requestId);
            BackgroundTransferService.Remove(request);
        }

        void OnBackgroundTransferRequestStatusChanged(object sender, BackgroundTransferEventArgs args)
        {
            BackgroundTransferRequest request = args.Request;
            int index = Int32.Parse(request.Tag);
            PictureInfo pictureInfo = pictureLibrary.Pictures[index];
            pictureInfo.DownloadStatus = FormatTransferStatus(request.TransferStatus);

            if (request.TransferStatus == TransferStatus.Completed)
                TransferCompleted(pictureInfo, request);
        }

        void TransferCompleted(PictureInfo pictureInfo, BackgroundTransferRequest request)
        {
            using (IsolatedStorageFile isoStorage = IsolatedStorageFile.GetUserStoreForApplication())
            {
                // Successful completion
                if (request.TransferError == null)
                {
                    // Move downloaded file to images directory
                    isoStorage.MoveFile(transferDirectory + "/" + pictureInfo.Filename,
                                        imagesDirectory + "/" + pictureInfo.Filename);

                    // Create thumbnail 
                    WriteableBitmap bitmap = null;

                    using (IsolatedStorageFileStream stream = isoStorage.OpenFile(imagesDirectory + "/" + pictureInfo.Filename, 
                                                                                  FileMode.Open, FileAccess.Read))
                    {
                        bitmap = PictureDecoder.DecodeJpeg(stream, 144, 144);
                    }

                    // Save thumbnail
                    using (IsolatedStorageFileStream stream = isoStorage.CreateFile(thumbnailsDirectory + "/" + pictureInfo.Filename))
                    {
                        bitmap.SaveJpeg(stream, bitmap.PixelWidth, bitmap.PixelHeight, 0, 80);
                    }
                }
                // Error (probably a Cancel so don't report it)
                else
                {
                    if (isoStorage.FileExists(transferDirectory + "/" + pictureInfo.Filename))
                        isoStorage.DeleteFile(transferDirectory + "/" + pictureInfo.Filename);
                }
            }

            pictureInfo.IsDownloading = false;
            pictureInfo.IsDownloaded = request.TransferError == null;

            if (BackgroundTransferService.Find(pictureInfo.DownloadRequestId) != null)
                BackgroundTransferService.Remove(request);
        }

        void OnBackgroundTransferRequestProgressChanged(object sender, BackgroundTransferEventArgs args)
        {
            BackgroundTransferRequest request = args.Request;
            int index = Int32.Parse(request.Tag);
            PictureInfo pictureInfo = pictureLibrary.Pictures[index];
            pictureInfo.DownloadProgress = 100.0 * request.BytesReceived / request.TotalBytesToReceive;
        }

        void ListBoxSelectionChanged(object sender, SelectionChangedEventArgs args)
        {
            if (listBox.SelectedItem != null)
            {
                PictureInfo pictureInfo = listBox.SelectedItem as PictureInfo;

                if (pictureInfo.IsDownloaded)
                {
                    string destination = String.Format("/ViewPage.xaml?Title={0}&Filename={1}", 
                                                       pictureInfo.Title,
                                                       imagesDirectory + "/" + pictureInfo.Filename);

                    this.NavigationService.Navigate(new Uri(destination, UriKind.Relative));
                }
                listBox.SelectedItem = null;
            }
        }

        void OnDeleteButtonClick(object sender, RoutedEventArgs args)
        {
            Button btn = sender as Button;
            PictureInfo pictureInfo = btn.DataContext as PictureInfo;

            MessageBoxResult result = MessageBox.Show("Delete " + pictureInfo.Title + "?",
                                                      "Big Picture", 
                                                       MessageBoxButton.OKCancel);
            if (result == MessageBoxResult.OK)
            {
                using (IsolatedStorageFile isoStorage = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    isoStorage.DeleteFile(imagesDirectory + "/" + pictureInfo.Filename);
                    isoStorage.DeleteFile(thumbnailsDirectory + "/" + pictureInfo.Filename);
                    pictureInfo.IsDownloaded = false;
                }
            }
        }

        void OnAppBarSettingsButtonClick(object sender, EventArgs args)
        {
            this.NavigationService.Navigate(new Uri("/TransferSettingsPage.xaml", UriKind.Relative));
        }

        string FormatTransferStatus(TransferStatus transferStatus)
        {
            switch (transferStatus)
            {
                case TransferStatus.None:
                    return "";

                case TransferStatus.WaitingForExternalPower:
                    return "Waiting for external power";

                case TransferStatus.WaitingForExternalPowerDueToBatterySaverMode:
                    return "Waiting for external power due to battery saver mode";

                case TransferStatus.WaitingForNonVoiceBlockingNetwork:
                    return "Waiting for non-voice-blocking network";

                case TransferStatus.WaitingForWiFi:
                    return "Waiting for WiFi";
            }
            return transferStatus.ToString();
        }
    }
}