﻿using System;
using System.Collections.Generic;
using System.IO;
using System.IO.IsolatedStorage;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using Microsoft.Devices.Sensors;
using Microsoft.Phone;
using Microsoft.Phone.Controls;
using Microsoft.Xna.Framework;

namespace BigPicture
{
    public partial class ViewPage : PhoneApplicationPage
    {
        Motion motion;
        double screenWidth, screenHeight, maxDimension;
        int imageWidth, imageHeight;
        string title, filename;
        double zoomInScale;
        double rotation;
        bool isZoomView;

        // Dependency property for zoom-in transition animation
        public static readonly DependencyProperty InterpolationFactorProperty =
            DependencyProperty.Register("InterpolationFactor",
                                        typeof(double),
                                        typeof(ViewPage),
                                        new PropertyMetadata(OnInterpolationFactorChanged));
        public ViewPage()
        {
            InitializeComponent();

            Loaded += OnLoaded;

            if (Motion.IsSupported)
            {
                motion = new Motion();
                motion.TimeBetweenUpdates = TimeSpan.FromMilliseconds(33);
                motion.CurrentValueChanged += OnMotionCurrentValueChanged;
            }
        }

        public double InterpolationFactor
        {
            set { SetValue(InterpolationFactorProperty, value); }
            get { return (double)GetValue(InterpolationFactorProperty); }
        }

        protected override void OnNavigatedTo(NavigationEventArgs args)
        {
            IDictionary<string, string> parameters = this.NavigationContext.QueryString;

            if (parameters.ContainsKey("Title"))
                title = parameters["Title"].ToUpper();

            if (parameters.ContainsKey("Filename"))
                filename = parameters["Filename"];

            // Start the Motion sensor if it's available
            if (motion != null)
            {
                try { motion.Start(); }
                catch { }
            }
            base.OnNavigatedTo(args);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs args)
        {
            if (motion != null)
            {
                motion.Stop();
            }
            base.OnNavigatedFrom(args);
        }

        void OnLoaded(object sender, RoutedEventArgs args)
        {
            // Save the screen dimensions
            screenWidth = this.ActualWidth;
            screenHeight = this.ActualHeight;
            maxDimension = Math.Max(screenWidth, screenHeight);

            // Initialize some values
            outlineBorder.Width = screenWidth;
            outlineBorder.Height = screenHeight;
            borderTransform.CenterX = screenWidth / 2;
            borderTransform.CenterY = screenHeight / 2;

            // Load the image from isolated storage
            if (image.Source == null && filename != null)
            {
                using (IsolatedStorageFile isoStorage = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    WriteableBitmap bitmap = null;

                    using (IsolatedStorageFileStream stream = isoStorage.OpenFile(filename, FileMode.Open, FileAccess.Read))
                    {
                        try
                        {
                            bitmap = PictureDecoder.DecodeJpeg(stream);
                        }
                        catch (Exception exc)
                        {
                            MessageBox.Show("Error creating bitmap: " + exc.Message, "Big Picture", MessageBoxButton.OK);
                        }
                    }
                    statusText.Visibility = Visibility.Collapsed;
                    image.Source = bitmap;

                    // Save image dimensions
                    imageWidth = bitmap.PixelWidth;
                    imageHeight = bitmap.PixelHeight;
                    titleText.Text = String.Format("{0} ({1}\x00D7{2})", title, imageWidth, imageHeight);

                    zoomInScale = Math.Min(screenWidth / imageWidth, screenHeight / imageHeight);
                    UpdateImageTransforms();
                }
            }
        }

        protected override void OnTap(GestureEventArgs args)
        {
            // Animate the InterpolationFactor property
            DoubleAnimation doubleAnimation = new DoubleAnimation
            {
                To = isZoomView ? 0 : 1,
            };
            Storyboard.SetTarget(doubleAnimation, this);
            Storyboard.SetTargetProperty(doubleAnimation,
                                         new PropertyPath(ViewPage.InterpolationFactorProperty));

            Storyboard storyboard = new Storyboard();
            storyboard.Children.Add(doubleAnimation);
            storyboard.Begin();
            isZoomView ^= true;
            base.OnTap(args);
        }

        void OnMotionCurrentValueChanged(object sender, SensorReadingEventArgs<MotionReading> args)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                // If the image isn't loaded yet, don't do anything
                if (image.Source == null)
                    return;

                // Get the rotation matrix & convert to horizontal coordinates
                Matrix matrix = args.SensorReading.Attitude.RotationMatrix;
	            HorizontalCoordinate horzCoord = HorizontalCoordinate.FromMotionMatrix(matrix);

                // Set the transform center on the Image element
                imageTransform.CenterX = (imageWidth + maxDimension) * 
                                            (180 + horzCoord.Azimuth) / 360 - maxDimension / 2;
                imageTransform.CenterY = (imageHeight + maxDimension) * 
                                            (90 - horzCoord.Altitude) / 180 - maxDimension / 2;

                // Set the translation on the Border element
                borderTransform.TranslateX = imageTransform.CenterX - screenWidth / 2;
                borderTransform.TranslateY = imageTransform.CenterY - screenHeight / 2;

                // Get rotation from Tilt
                rotation = -horzCoord.Tilt;
                UpdateImageTransforms();
            });
        }

        static void OnInterpolationFactorChanged(DependencyObject obj, 
                                                 DependencyPropertyChangedEventArgs args)
        {
            (obj as ViewPage).UpdateImageTransforms();
        }

        void UpdateImageTransforms()
        {
            // If being zoomed out, set scaling
            double interpolatedScale = 1 + InterpolationFactor * (zoomInScale - 1);
            imageTransform.ScaleX =
            imageTransform.ScaleY = interpolatedScale;

            // Move transform center to screen center
            imageTransform.TranslateX = screenWidth / 2 - imageTransform.CenterX;
            imageTransform.TranslateY = screenHeight / 2 - imageTransform.CenterY;

            // If being zoomed out, adjust for scaling
            imageTransform.TranslateX -= InterpolationFactor * 
                                (screenWidth / 2 - zoomInScale * imageTransform.CenterX);
            imageTransform.TranslateY -= InterpolationFactor * 
                                (screenHeight / 2 - zoomInScale * imageTransform.CenterY);

            // If being zoomed out, center image in screen
            imageTransform.TranslateX += InterpolationFactor * 
                                (screenWidth - zoomInScale * imageWidth) / 2;
            imageTransform.TranslateY += InterpolationFactor * 
                                (screenHeight - zoomInScale * imageHeight) / 2;

            // Set border thickness
            outlineBorder.BorderThickness = new Thickness(2 / interpolatedScale);
            outlineRectangle.StrokeThickness = 2 / interpolatedScale;

            // Set rotation on image and border
            imageTransform.Rotation = (1 - InterpolationFactor) * rotation;
            borderTransform.Rotation = -rotation;
        }
    }
}